package com.laguerre.kan;

public final class RenderingSettings {
	public static enum ShadowSetting {
		HARD_AND_SOFT,
		HARD_ONLY,
		NO_SHADOW
	}

	public static enum ShadowResolution {
		LOW,
		MEDIUM,
		HIGH,
		VERY_HIGH
	}

	private static RenderingSettings instance = null;
	public static RenderingSettings getInstance() {
		if (instance == null) {
			instance = new RenderingSettings();
		}
		return instance;
	}
	public static void resetInstance() {
		instance = null;
	}

	private ShadowSetting shadowSetting = ShadowSetting.HARD_AND_SOFT;
	private ShadowResolution shadowResolution = ShadowResolution.MEDIUM;
	private float shadowMaxDistance = 10f;
	private final Color ambientLight = Color.getNewBlack();
	private final Vector3 gravity = new Vector3(0.0f, -9.81f, 0.0f);

	private RenderingSettings() {
	}

	public void setShadowSetting(ShadowSetting shadowSetting) {
		this.shadowSetting = shadowSetting;
	}

	public ShadowSetting getShadowSetting() {
		return this.shadowSetting;
	}

	public void setShadowResolution(ShadowResolution shadowResolution) {
		this.shadowResolution = shadowResolution;
	}

	public ShadowResolution getShadowResolution() {
		return this.shadowResolution;
	}

	public void setShadowMaxDistance(float shadowMaxDistance) {
		this.shadowMaxDistance = shadowMaxDistance;
	}

	public float getShadowMaxDistance() {
		return this.shadowMaxDistance;
	}

	public void setAmbientLight(Color c) {
		this.ambientLight.setValues(c);
	}

	public Color getSharedAmbientLight() {
		return this.ambientLight;
	}

	public void setGravity(float x, float y, float z) {
		this.gravity.setValues(x, y, z);
	}

	public void setGravity(Vector3 g) {
		this.gravity.setValues(g);
	}

	public Vector3 getSharedGravity() {
		return this.gravity;
	}
}
