package com.laguerre.kan;

import android.opengl.GLES20;

public abstract class Texture {
	private static int textureCounter = 0;

	public static void resetTextureCounter() {
		textureCounter = 0;
	}
	public static int getThenIncrTextureCounter() {
		final int n = textureCounter;
		textureCounter++;
		return n;
	}

	public static class HasBeenSetAfterCreationException extends RuntimeException {
		private static final long serialVersionUID = 2463777814088827353L;

		public HasBeenSetAfterCreationException(String msg) {
			super(msg);
		}
	}

	public static enum MinifyingFilter {
		LINEAR,
		NEAREST
	}

	public static enum MagnificationFilter {
		LINEAR,
		NEAREST
	}

	public static enum Wrapping_S {
		CLAMP_TO_EDGE,
		REPEAT,
		MIRRORED_REPEAT
	}

	public static enum Wrapping_T {
		CLAMP_TO_EDGE,
		REPEAT,
		MIRRORED_REPEAT
	}

	private static int maxCombinedTextures = -1;
	protected final int[] textureHandle = { -1 };
	protected MinifyingFilter minFilter = MinifyingFilter.NEAREST;
	protected MagnificationFilter magFilter = MagnificationFilter.NEAREST;
	protected Wrapping_S wrappingS = Wrapping_S.CLAMP_TO_EDGE;
	protected Wrapping_T wrappingT = Wrapping_T.CLAMP_TO_EDGE;

	public abstract int getWidth();

	public abstract int getHeight();

	public final void setMinifyingFilter(MinifyingFilter minFilter) {
		checkTextureNotCreated();
		this.minFilter = minFilter;
	}

	public final MinifyingFilter getMinifyingFilter() {
		return this.minFilter;
	}

	public final void setMagnificationFilter(MagnificationFilter magFilter) {
		checkTextureNotCreated();
		this.magFilter = magFilter;
	}

	public final MagnificationFilter getMagnificationFilter() {
		return this.magFilter;
	}

	public final void setTextureWrapping_S(Wrapping_S wrappingS) {
		checkTextureNotCreated();
		this.wrappingS = wrappingS;
	}

	public final Wrapping_S getTextureWrapping_S() {
		return this.wrappingS;
	}

	public final void setTextureWrapping_T(Wrapping_T wrappingT) {
		checkTextureNotCreated();
		this.wrappingT = wrappingT;
	}

	public final Wrapping_T getTextureWrapping_T() {
		return this.wrappingT;
	}

	protected void checkTextureNotCreated() {
		if (this.textureHandle[0] > 0) {
			throw new HasBeenSetAfterCreationException(
					"A \"set\" function for texture params has been called after its creation.");
		}
	}

	public abstract void create();

	protected final void doCreation() {
		GLES20.glGenTextures(1, this.textureHandle, 0);

		if (this.textureHandle[0] > 0) {
			// Bind to the texture in OpenGL
			GLES20.glBindTexture(GLES20.GL_TEXTURE_2D, this.textureHandle[0]);

			// Set filtering
			boolean mipmap = false;
			if (mipmap) {
				GLES20.glTexParameterf(GLES20.GL_TEXTURE_2D, GLES20.GL_TEXTURE_MIN_FILTER, GLES20.GL_LINEAR_MIPMAP_LINEAR);
			} else {
				GLES20.glTexParameterf(GLES20.GL_TEXTURE_2D, GLES20.GL_TEXTURE_MIN_FILTER, getMinifyingFilterParam(this.minFilter));
			}
			GLES20.glTexParameteri(GLES20.GL_TEXTURE_2D, GLES20.GL_TEXTURE_MAG_FILTER, getMagnificationFilterParam(this.magFilter));

			GLES20.glTexParameteri(GLES20.GL_TEXTURE_2D, GLES20.GL_TEXTURE_WRAP_S, getWrapping_SParam(this.wrappingS));
			GLES20.glTexParameteri(GLES20.GL_TEXTURE_2D, GLES20.GL_TEXTURE_WRAP_T, getWrapping_TParam(this.wrappingT));

			if (mipmap) {
				GLES20.glGenerateMipmap(GLES20.GL_TEXTURE_2D);
			}
		} else {
			throw new RuntimeException("Error loading texture.");
		}
	}

	public final int getTextureHandle() {
		return this.textureHandle[0];
	}

	public void freeResources() {
		if (this.textureHandle[0] > 0) {
			GLES20.glDeleteTextures(1, this.textureHandle, 0);
			this.textureHandle[0] = -1;
		}
	}

	protected int getMinifyingFilterParam(MinifyingFilter minFilter) {
		int ret = GLES20.GL_NEAREST;
		switch (minFilter) {
		case LINEAR:
			ret = GLES20.GL_LINEAR;
			break;
		case NEAREST:
		default:
			break;
		}
		return ret;
	}

	protected int getMagnificationFilterParam(MagnificationFilter maxFilter) {
		int ret = GLES20.GL_NEAREST;
		switch (maxFilter) {
		case LINEAR:
			ret = GLES20.GL_LINEAR;
			break;
		case NEAREST:
		default:
			break;
		}
		return ret;
	}

	protected int getWrapping_SParam(Wrapping_S wrapping) {
		int ret = GLES20.GL_CLAMP_TO_EDGE;
		switch (wrapping) {
		case REPEAT:
			ret = GLES20.GL_REPEAT;
			break;
		case MIRRORED_REPEAT:
			ret = GLES20.GL_MIRRORED_REPEAT;
			break;
		case CLAMP_TO_EDGE:
		default:
			break;
		}
		return ret;
	}

	protected int getWrapping_TParam(Wrapping_T wrapping) {
		int ret = GLES20.GL_CLAMP_TO_EDGE;
		switch (wrapping) {
		case REPEAT:
			ret = GLES20.GL_REPEAT;
			break;
		case MIRRORED_REPEAT:
			ret = GLES20.GL_MIRRORED_REPEAT;
			break;
		case CLAMP_TO_EDGE:
		default:
			break;
		}
		return ret;
	}

	public static int getGLTexNum(int texNum) {
		int ret = GLES20.GL_TEXTURE0;
		switch (texNum) {
		case 0:
			ret = GLES20.GL_TEXTURE0;
			break;
		case 1:
			ret = GLES20.GL_TEXTURE1;
			break;
		case 2:
			ret = GLES20.GL_TEXTURE2;
			break;
		case 3:
			ret = GLES20.GL_TEXTURE3;
			break;
		case 4:
			ret = GLES20.GL_TEXTURE4;
			break;
		case 5:
			ret = GLES20.GL_TEXTURE5;
			break;
		case 6:
			ret = GLES20.GL_TEXTURE6;
			break;
		case 7:
			ret = GLES20.GL_TEXTURE7;
			break;
		case 8:
			ret = GLES20.GL_TEXTURE8;
			break;
		case 9:
			ret = GLES20.GL_TEXTURE9;
			break;
		case 10:
			ret = GLES20.GL_TEXTURE10;
			break;
		case 11:
			ret = GLES20.GL_TEXTURE11;
			break;
		case 12:
			ret = GLES20.GL_TEXTURE12;
			break;
		case 13:
			ret = GLES20.GL_TEXTURE13;
			break;
		case 14:
			ret = GLES20.GL_TEXTURE14;
			break;
		case 15:
			ret = GLES20.GL_TEXTURE15;
			break;
		default:
			break;
		}
		return ret;
	}

	public static int getMaxCombinedTextures() {
		if (maxCombinedTextures == -1) {
			final int[] ret = new int[1];
			GLES20.glGetIntegerv(GLES20.GL_MAX_COMBINED_TEXTURE_IMAGE_UNITS, ret, 0);
			maxCombinedTextures = ret[0];
		}
		return maxCombinedTextures;
	}
}
