package com.laguerre.mytest;

import javax.microedition.khronos.egl.EGLConfig;
import javax.microedition.khronos.opengles.GL10;

import com.laguerre.kan.AndroidGL20Compact;
import com.laguerre.kan.Camera;
import com.laguerre.kan.Color;
import com.laguerre.kan.ImageEffects;
import com.laguerre.kan.Light;
import com.laguerre.kan.Lights;
import com.laguerre.kan.Material;
import com.laguerre.kan.Mesh;
import com.laguerre.kan.MeshImporter;
import com.laguerre.kan.MeshRenderer;
import com.laguerre.kan.ObjImporter;
import com.laguerre.kan.ParticleSystem;
import com.laguerre.kan.ParticleSystemRenderer;
import com.laguerre.kan.PassShader;
import com.laguerre.kan.Quaternion;
import com.laguerre.kan.RenderTexture;
import com.laguerre.kan.RenderingSettings;
import com.laguerre.kan.SceneConductor;
import com.laguerre.kan.SceneObject;
import com.laguerre.kan.Shader;
import com.laguerre.kan.SurfaceInfo;
import com.laguerre.kan.Texture;
import com.laguerre.kan.Texture2D;
import com.laguerre.kan.Utils;
import com.laguerre.kan.Vector3;
import com.laguerre.kan.Light.LightType;
import com.laguerre.kan.Light.ShadowCastingType;
import com.laguerre.kan.ParticleSystem.Burst;
import com.laguerre.kan.RenderingSettings.ShadowResolution;

import android.content.Context;
import android.graphics.Bitmap;
import android.opengl.GLSurfaceView;

public class MyGL20Renderer implements GLSurfaceView.Renderer {
	private Context context;

	public MyGL20Renderer(Context context) {
		this.context = context;
		// To be accessible later, somewhere else...
		Utils.setContext(context);
	}

	@Override
    public void onSurfaceCreated(GL10 unused, EGLConfig config) {
		AndroidGL20Compact.resetInstance();
		AndroidGL20Compact.getInstance();

		SurfaceInfo.resetInstance();
		RenderingSettings.resetInstance();
		Lights.resetInstance();
		SceneConductor.resetInstance();

		PassShader.setExceptionOnCompilationFailed(false);
		PassShader.setExceptionOnLinkingFailed(false);

		buildWorld();
	}

	private SceneObject camSob0;

	private SceneObject cubeCam0;
	private SceneObject camSob1;

	private void buildWorld() {
		RenderingSettings.getInstance().setAmbientLight(new Color(0.1f, 0.1f, 0.1f, 1f));
		RenderingSettings.getInstance().setShadowMaxDistance(20f);
		RenderingSettings.getInstance().setShadowResolution(ShadowResolution.LOW);

		// Layer 1  (i.e. a camera will inject what will be see in this layer to the layer).
		//
//		final int layer1 = 1 << 1;
//		final SceneObject myCamera1 = SceneConductor.getInstance().addSceneObject("Camera 1");
//		myCamera1.getSharedTransform().translateGlobally(-0.6f, 0f, 1f);
//		myCamera1.addComponent(Camera.class);
//		myCamera1.addComponent(PerlinImageEffect.class);
//		myCamera1.getSharedCamera().setCullingMask(layer1);
//		this.camSob1 = myCamera1;
//
//		addQuad(new Vector3(-0.5f, 0.2f, -4f)).setLayer(layer1);
//
//		final SceneObject lightSobQuad = SceneConductor.getInstance().addSceneObject("Light quad");
//		lightSobQuad.setLayer(layer1);
//		lightSobQuad.getSharedTransform().translateGlobally(0f, 0.2f, -3f);
//		final Light lightQuad = lightSobQuad.addComponent(Light.class);
//		lightQuad.setLightType(Light.LightType.POINT);
//		lightQuad.setConstantAttenuation(1.0f);
//		lightQuad.setLinearAttenuation(0.15f);
//		lightQuad.setQuadraticAttenuation(0.8f);
//		// Very red to see the quad once projected on the cube of the camera0.
//		lightQuad.setDiffuseColor(0.8f, 0.0f, 0.0f, 0.8f);
//		lightQuad.setSpecularColor(0.9f, 0.0f, 0.0f, 0.9f);


		// Now, layer 0 (it will be rendered on screen)
		//
		final SceneObject parentMyCamera0 = SceneConductor.getInstance().addSceneObject("Parent Camera 0");
		final SceneObject myCamera0 = SceneConductor.getInstance().addSceneObject("Camera 0");
		myCamera0.getSharedTransform().setParent(parentMyCamera0.getSharedTransform());
//		parentMyCamera0.getSharedTransform().translateGlobally(2.6f, 1.6f, 1.0f);
//		final Quaternion qc = new Quaternion(-50f, Vector3.RIGHT);
//		qc.mulSelf(new Quaternion(40f, Vector3.UP));
//		myCamera0.getSharedTransform().rotateLocally(qc);
		parentMyCamera0.getSharedTransform().translateGlobally(0.3f, 0f, 6f);
		//myCamera0.getSharedTransform().rotateLocally(new Quaternion(90f, Vector3.RIGHT));
		myCamera0.addComponent(Camera.class);
		// Apply a post-effect.
		//myCamera0.addComponent(PerlinImageEffect.class);
		//myCamera0.addComponent(GaussianBlurImageEffect.class);
		//myCamera0.addComponent(LaplaceEdgeImageEffect.class);
		myCamera0.getSharedCamera().setNearAndFarClipPlanes(-0.3f, -20f);
		// Makes this camera, the camera that renders to screen.
		myCamera0.getSharedCamera().setAsRenderingOnScreen();
		this.camSob0 = myCamera0;


		// Replacement shader example.
//		final Shader replacementShader = new Shader();
//		final PassShader passReplacementShader0 = replacementShader.addPassShader(0);
//		passReplacementShader0.createProgram();
//		passReplacementShader0.attachVertexShader(Utils.readTextFileFromRawResource(
//				this.context, R.raw.write_depth_shadow_mapping_v_glsl));
//		passReplacementShader0.attachFragmentShader(Utils.readTextFileFromRawResource(
//				this.context, R.raw.write_depth_shadow_mapping_f_glsl));
//		passReplacementShader0.linkProgram();
//		myCamera0.getSharedCamera().setReplacementShader(replacementShader,
//				Shader.CommonShaderTags.OPAQUE);


		//this.cubeCam0 = addDragonCharacter(new Vector3(0.5f, 0.2f, -1.5f));
		this.cubeCam0 = addDragonCharacter(new Vector3(0.0f, 0.0f, -0.5f));
		//this.cubeCam0 = addCube(new Vector3(0.5f, 0.2f, -1.5f));


		//addQuad(new Vector3(0.0f, -0.4f, 1.3f), new Quaternion(-50f, Vector3.RIGHT));
		addQuad(new Vector3(0.0f, -0.2f, -1.1f), new Quaternion(-20f, Vector3.RIGHT));

//		final SceneObject lightSob0 = SceneConductor.getInstance().addSceneObject("Light 0");
//		lightSob0.getSharedTransform().translateGlobally(0.5f, 0.2f, 1f);
//		final Light light0 = lightSob0.addComponent(Light.class);
//		light0.setLightType(Light.LightType.SPOT);
//		light0.setDiffuseColor(0.5f, 0.5f, 0.4f, 0.7f);
//		light0.setSpecularColor(0.8f, 0.8f, 0.75f, 0.8f);
//		light0.setRange(-1f);
//		light0.setConstantAttenuation(0f);
//		light0.setLinearAttenuation(0f);
//		light0.setQuadraticAttenuation(0.1f);
//		light0.setInnerCutOffAngle(2f);
//		light0.setOuterCutOffAngle(10f);

		final SceneObject parentLightSob1 = SceneConductor.getInstance().addSceneObject("Parent Light 1");
		final SceneObject lightSob1 = SceneConductor.getInstance().addSceneObject("Light 1");
		lightSob1.getSharedTransform().setParent(parentLightSob1.getSharedTransform());
		parentLightSob1.getSharedTransform().translateGlobally(2.6f, 1.6f, 1.0f);
		Quaternion ql1 = new Quaternion(-50f, Vector3.RIGHT);
		ql1.mulSelf(new Quaternion(40f, Vector3.UP));
		lightSob1.getSharedTransform().rotateLocally(ql1);
		final Light light1 = lightSob1.addComponent(Light.class);
		light1.setLightType(Light.LightType.DIRECTIONAL);
		light1.setQuadraticAttenuation(0.3f);
		light1.setDiffuseColor(0.3f, 0.3f, 0.35f, 0.3f);
		light1.setSpecularColor(0.4f, 0.4f, 0.4f, 0.6f);
		//light1.setShadow(ShadowCastingType.SOFT);

		final Shader light1ReplacementShader = new Shader();
		final PassShader passLight1ReplacementShader0 = light1ReplacementShader.addPassShader(0);
		passLight1ReplacementShader0.createProgram();
		passLight1ReplacementShader0.attachVertexShader(Utils.readTextFileFromRawResource(
				this.context, R.raw.write_depth_shadow_mapping_v_glsl));
		passLight1ReplacementShader0.attachFragmentShader(Utils.readTextFileFromRawResource(
				this.context, R.raw.write_depth_shadow_mapping_f_glsl));
		passLight1ReplacementShader0.linkProgram();
		light1.setShadowReplacementShader(light1ReplacementShader);

		/*
		final Shader light1PostEffectShader = new Shader();
		final PassShader passLight1PostEffectShader0 = light1PostEffectShader.addPassShader(0);
		passLight1PostEffectShader0.createProgram();
		passLight1PostEffectShader0.attachVertexShader(ImageEffects.getBasicVertexShaderCode());
		passLight1PostEffectShader0.attachFragmentShader(Utils.readTextFileFromRawResource(
				this.context, R.raw.gaussian_blur_f_glsl));
		passLight1PostEffectShader0.linkProgram();
		light1.setShadowAlternativePostEffectShader(light1PostEffectShader);
		*/
		//lightSob1.addComponent(GaussianBlurImageEffect.class);
	}

	private SceneObject addTriangle(Vector3 translation) {
		final SceneObject sob = SceneConductor.getInstance().addSceneObject("Triangle");
		sob.getSharedTransform().translateGlobally(translation);

		final SimpleTriangle mesh = sob.addComponent(SimpleTriangle.class, "Simple triangle");
		MeshRenderer sobRenderer = sob.addComponent(MeshRenderer.class, "Mesh renderer");

		final Material sobMat = new Material();
		sobMat.setMaterialName("triangle-mat");
		final Shader shader = new Shader();
		sobMat.setShader(shader);
		SceneConductor.getInstance().addSceneObjectToRenderQueue(
				sob, Shader.RenderQueueType.GEOMETRY.index());
		final PassShader passShader0 = shader.addPassShader(0);
		passShader0.setDepthTest(PassShader.DepthTest.GEQUAL);
		passShader0.createProgram();
		passShader0.attachVertexShader(Utils.readTextFileFromRawResource(
				this.context, R.raw.one_tex_v_glsl));
		passShader0.attachFragmentShader(Utils.readTextFileFromRawResource(
				this.context, R.raw.one_tex_f_glsl));
		passShader0.linkProgram();

		final Texture2D mainTex = new Texture2D();
		mainTex.setBitmap(this.context, R.drawable.carte_mcdo, Bitmap.Config.ARGB_8888);
		mainTex.create();
		sobMat.setShaderTexture("mainTex", mainTex);
		sobMat.setShaderVector("diffuseColor", Color.WHITE);
		sobMat.setShaderVector("specularColor", Color.WHITE);
		sobMat.setShaderValue("shininess", 5f);

		sobRenderer.setMaterial(sobMat);

		mesh.init();

		return sob;
	}

	private SceneObject addQuad(Vector3 translation, Quaternion rotation) {
		final SceneObject parentSob = SceneConductor.getInstance().addSceneObject("Parent of Quad");
		final SceneObject sob = SceneConductor.getInstance().addSceneObject("Quad");
		sob.getSharedTransform().setParent(parentSob.getSharedTransform());

		parentSob.getSharedTransform().translateGlobally(translation);
		sob.getSharedTransform().rotateLocally(rotation);

		final SimpleQuad mesh = sob.addComponent(SimpleQuad.class, "Simple quad");
		MeshRenderer sobRenderer = sob.addComponent(MeshRenderer.class, "Mesh renderer");
		sobRenderer.setReceiveShadows(true);

		final Material sobMat = new Material();
		sobMat.setMaterialName("quad-material");
		final Shader shader = new Shader();
		sobMat.setShader(shader);
		SceneConductor.getInstance().addSceneObjectToRenderQueue(
				sob, Shader.RenderQueueType.GEOMETRY.index());
		final PassShader passShader0 = shader.addPassShader(0);
		passShader0.setDepthTest(PassShader.DepthTest.GEQUAL);
		passShader0.createProgram();
		passShader0.attachVertexShader(Utils.readTextFileFromRawResource(
				this.context, R.raw.one_tex_v_glsl));
		passShader0.attachFragmentShader(Utils.readTextFileFromRawResource(
				this.context, R.raw.one_tex_f_glsl));
		passShader0.linkProgram();

		final Texture2D mainTex = new Texture2D();
		mainTex.setBitmap(this.context, R.drawable.carte_mcdo, Bitmap.Config.ARGB_8888);
		mainTex.create();
		sobMat.setShaderTexture("mainTex", mainTex);
		sobMat.setShaderVector("diffuseColor", Color.WHITE);
		sobMat.setShaderVector("specularColor", Color.WHITE);
		sobMat.setShaderValue("shininess", 5f);

		sobRenderer.setMaterial(sobMat);

		mesh.init();

		return sob;
	}

	private SceneObject addCube(Vector3 translation) {
		final SceneObject sobParent = SceneConductor.getInstance().addSceneObject("Cube parent");
		sobParent.getSharedTransform().translateGlobally(translation);

		final SceneObject sobCube = SceneConductor.getInstance().addSceneObject("Cube");
		sobCube.getSharedTransform().setParent(sobParent.getSharedTransform());

		final Quaternion q = new Quaternion(30f, Vector3.RIGHT);
		q.mulSelf(new Quaternion(15f, Vector3.UP));
		sobCube.getSharedTransform().rotateLocally(q);

		final Cube mesh = sobCube.addComponent(Cube.class, "Cube");
		MeshRenderer sobRenderer = sobCube.addComponent(MeshRenderer.class, "Mesh renderer");

		final Material sobMat = new Material();
		sobMat.setMaterialName("cube-material");
		final Shader shader = new Shader();
		sobMat.setShader(shader);
		SceneConductor.getInstance().addSceneObjectToRenderQueue(
				sobCube, Shader.RenderQueueType.GEOMETRY.index());
		final PassShader passShader0 = shader.addPassShader(0);
		passShader0.setDepthTest(PassShader.DepthTest.GEQUAL);
		passShader0.createProgram();

		passShader0.attachVertexShader(Utils.readTextFileFromRawResource(
				this.context, R.raw.one_tex_v_glsl));
		passShader0.attachFragmentShader(Utils.readTextFileFromRawResource(
				this.context, R.raw.one_tex_f_glsl));

//		passShader0.attachVertexShader(Utils.readTextFileFromRawResource(
//				this.context, R.raw.normal_mapping_v_glsl));
//		passShader0.attachFragmentShader(Utils.readTextFileFromRawResource(
//				this.context, R.raw.normal_mapping_f_glsl));
//		passShader0.linkProgram();

		final Texture2D mainTex = new Texture2D();
		mainTex.setBitmap(this.context, R.drawable.carte_mcdo, Bitmap.Config.ARGB_8888);
		mainTex.create();
		final Texture2D normalTex = new Texture2D();
		normalTex.setBitmap(this.context, R.drawable.carte_mcdo_normals, Bitmap.Config.ARGB_8888);
		normalTex.create();
		sobMat.setShaderTexture("mainTex", mainTex);
		sobMat.setShaderTexture("normalTex", normalTex);
		sobMat.setShaderVector("diffuseColor", Color.WHITE);
		sobMat.setShaderVector("specularColor", Color.WHITE);
		sobMat.setShaderValue("shininess", 5f);

		sobRenderer.setMaterial(sobMat);

		mesh.init();

		return sobCube;
	}

	private SceneObject addDragonCharacter(Vector3 translation) {
		final SceneObject sobParent = SceneConductor.getInstance().addSceneObject("Dragon parent");
		final Quaternion q = new Quaternion(110f, Vector3.FORWARD);
		q.mulSelf(new Quaternion(-30f, Vector3.UP));
		sobParent.getSharedTransform().rotateLocally(q);
		sobParent.getSharedTransform().translateGlobally(translation);

		final SceneObject sobDragon = SceneConductor.getInstance().addSceneObject("Dragon");
		sobDragon.getSharedTransform().setParent(sobParent.getSharedTransform());
		MeshRenderer sobRenderer = sobDragon.addComponent(MeshRenderer.class, "Mesh renderer");
		sobRenderer.setReceiveShadows(false);

		final MeshImporter meshImporter = new MeshImporter();
		meshImporter.setGlobalScale(0.06f);
		//meshImporter.setTangentImportMode(MeshImporter.ImportNormalTangentMode.CALCULATE);
		meshImporter.importToTarget(
				Utils.getInputStreamFromRawResource(R.drawable.dragon3d_dragon),
				sobDragon, ObjImporter.class);

		final Mesh mesh = sobDragon.getAnyComponent(Mesh.class);

		final Shader shader = new Shader();
		SceneConductor.getInstance().addSceneObjectToRenderQueue(
				sobDragon, Shader.RenderQueueType.GEOMETRY.index());
		final PassShader passShader0 = shader.addPassShader(0);
		passShader0.createProgram();

//		passShader0.attachVertexShader(Utils.readTextFileFromRawResource(
//				this.context, R.raw.simple_tex_v_glsl));
//		passShader0.attachFragmentShader(Utils.readTextFileFromRawResource(
//				this.context, R.raw.simple_tex_f_glsl));

		passShader0.attachVertexShader(Utils.readTextFileFromRawResource(
				this.context, R.raw.one_tex_v_glsl));
		passShader0.attachFragmentShader(Utils.readTextFileFromRawResource(
				this.context, R.raw.one_tex_f_glsl));

//		passShader0.attachVertexShader(Utils.readTextFileFromRawResource(
//				this.context, R.raw.normal_mapping_v_glsl));
//		passShader0.attachFragmentShader(Utils.readTextFileFromRawResource(
//				this.context, R.raw.normal_mapping_f_glsl));

		passShader0.linkProgram();

		final Texture2D whiteTex = new Texture2D();
		whiteTex.setBitmap(this.context, R.drawable.white, Bitmap.Config.ARGB_8888);
		whiteTex.create();

		final Texture2D pinkTex = new Texture2D();
		pinkTex.setBitmap(this.context, R.drawable.pink, Bitmap.Config.ARGB_8888);
		pinkTex.create();

		final Texture2D default_0x_Tex = new Texture2D();
		default_0x_Tex.setBitmap(this.context, R.drawable.dragon3d_203, Bitmap.Config.ARGB_8888);
		default_0x_Tex.create();

		final Texture2D default_00_Tex = new Texture2D();
		default_00_Tex.setBitmap(this.context, R.drawable.dragon3d_201, Bitmap.Config.ARGB_8888);
		default_00_Tex.create();

		final Texture2D default_01_Tex = new Texture2D();
		default_00_Tex.setBitmap(this.context, R.drawable.dragon3d_200, Bitmap.Config.ARGB_8888);
		default_00_Tex.create();

		final Texture2D default_02_Tex = new Texture2D();
		default_00_Tex.setBitmap(this.context, R.drawable.dragon3d_202, Bitmap.Config.ARGB_8888);
		default_00_Tex.create();

		for (int i = 0, iEnd = mesh.getNbOfSubMeshes(); i < iEnd; i++) {
			final Material mat = sobRenderer.getSharedMaterial(i);
			if (mat != null) {
				final String materialName = mat.getMaterialName();

				if (materialName.equals("_5___Default")) {
					final Shader subShader = new Shader();
					mat.setShader(subShader);
					final PassShader passSubShader0 = subShader.addPassShader(0);
					passSubShader0.setDepthTest(PassShader.DepthTest.GEQUAL);
					passSubShader0.useExistingProgram(passShader0.getProgramHandle());

					mat.setShaderTexture("mainTex", default_0x_Tex);
					mat.setShaderVector("diffuseColor", new Color(1.0f, 1.0f, 1.0f, 1.0f));
					mat.setShaderVector("specularColor", new Color(0.9f, 0.9f, 0.9f, 1.0f));
					mat.setShaderValue("shininess", 20.960784f);
				} else if (materialName.equals("_5___Default0")) {
					final Shader subShader = new Shader();
					mat.setShader(subShader);
					final PassShader passSubShader0 = subShader.addPassShader(0);
					passSubShader0.setDepthTest(PassShader.DepthTest.GEQUAL);
					passSubShader0.useExistingProgram(passShader0.getProgramHandle());

					mat.setShaderTexture("mainTex", default_00_Tex);
					mat.setShaderVector("diffuseColor", new Color(0.8f, 0.8f, 0.8f, 1.0f));
					mat.setShaderVector("specularColor", new Color(0.9f, 0.9f, 0.9f, 1.0f));
					mat.setShaderValue("shininess", 20.960784f);
				} else if (materialName.equals("_5___Default1")) {
					final Shader subShader = new Shader();
					mat.setShader(subShader);
					final PassShader passSubShader0 = subShader.addPassShader(0);
					passSubShader0.setDepthTest(PassShader.DepthTest.GEQUAL);
					passSubShader0.useExistingProgram(passShader0.getProgramHandle());

					mat.setShaderTexture("mainTex", default_01_Tex);
					mat.setShaderVector("diffuseColor", new Color(0.8f, 0.8f, 0.8f, 1.0f));
					mat.setShaderVector("specularColor", new Color(0.9f, 0.9f, 0.9f, 1.0f));
					mat.setShaderValue("shininess", 20.960784f);
				} else if (materialName.equals("_5___Default2")) {
					final Shader subShader = new Shader();
					mat.setShader(subShader);
					final PassShader passSubShader0 = subShader.addPassShader(0);
					passSubShader0.setDepthTest(PassShader.DepthTest.GEQUAL);
					passSubShader0.useExistingProgram(passShader0.getProgramHandle());

					mat.setShaderTexture("mainTex", default_02_Tex);
					mat.setShaderVector("diffuseColor", new Color(0.8f, 0.8f, 0.8f, 1.0f));
					mat.setShaderVector("specularColor", new Color(0.9f, 0.9f, 0.9f, 1.0f));
					mat.setShaderValue("shininess", 20.960784f);
				} else {
					mat.setShader(shader);

					mat.setShaderTexture("mainTex", pinkTex);
					mat.setShaderVector("diffuseColor", Color.WHITE);
					mat.setShaderVector("specularColor", Color.WHITE);
					mat.setShaderValue("shininess", 1f);
				}
			}
		}

		final SceneObject particleSob = SceneConductor.getInstance().addSceneObject("Particles for dragon fire");
		particleSob.getSharedTransform().setParent(sobDragon.getSharedTransform());
		particleSob.getSharedTransform().translateLocally(0.8f, -0.075f, 0.8f);
		final ParticleSystemRenderer particleRenderer = particleSob.addComponent(ParticleSystemRenderer.class);
		particleRenderer.setCastShadows(false);
		final Material particleMat = new Material();
		particleMat.setMaterialName("particle");
		particleRenderer.setMaterial(particleMat, 0);
		final Shader shaderMatParticle = new Shader();
		shaderMatParticle.setTagValue("RenderType", Shader.CommonShaderTags.TRANSPARENT);
		particleMat.setShader(shaderMatParticle);
		SceneConductor.getInstance().addSceneObjectToRenderQueue(
				particleSob, Shader.RenderQueueType.TRANSPARENT.index());
		final PassShader passShader0MatParticle = shaderMatParticle.addPassShader(0);
		passShader0MatParticle.setDepthWriting(false);
		passShader0MatParticle.setBlending(true);
		passShader0MatParticle.setBlendFunc(PassShader.BlendSrc.ALPHA, PassShader.BlendDst.ALPHA);
		passShader0MatParticle.createProgram();
		passShader0MatParticle.attachVertexShader(Utils.readTextFileFromRawResource(
				this.context, R.raw.simple_particle_v_glsl));
		passShader0MatParticle.attachFragmentShader(Utils.readTextFileFromRawResource(
				this.context, R.raw.simple_particle_f_glsl));
		passShader0MatParticle.linkProgram();
		final Texture2D mainTexParticle = new Texture2D();
		mainTexParticle.setBitmap(this.context, R.drawable.round_particle, Bitmap.Config.ARGB_8888);
		mainTexParticle.setMinifyingFilter(Texture.MinifyingFilter.LINEAR);
		mainTexParticle.setMagnificationFilter(Texture.MagnificationFilter.LINEAR);
		mainTexParticle.create();
		particleMat.setShaderTexture("mainTex", mainTexParticle);
		final ParticleSystem particleSystem = particleSob.addComponent(ParticleSystem.class);

		final ParticleSystem.ConeShape cone = new ParticleSystem.ConeShape();
		cone.setUnitConeAngle(30f);
		cone.setEmitFrom(ParticleSystem.ConeShape.EmitFrom.ORIGIN);
		particleSystem.setEmitterShape(cone);

		final ParticleSystem.ApplyVelocityModule applyVelocityModule = new ParticleSystem.ApplyVelocityModule();
		applyVelocityModule.setApplyGravity(false);
		particleSystem.addParticleModule(applyVelocityModule);
		final ParticleSystem.UpdateColorOverTimeModule colorOverTime = new ParticleSystem.UpdateColorOverTimeModule();
		particleSystem.addParticleModule(colorOverTime);
		colorOverTime.setInitialColor(1f, 0f, 0f, 1f);
		colorOverTime.setFinalColor(0.2f, 0.6f, 0.9f, 0f);

		particleSystem.setStartLifeTime(4f);
		particleSystem.setRateEmission(20);
		particleSystem.setStartSize(0.15f);
		particleSystem.setStartRotation(-30f);
		particleSystem.setStartSpeed(0.6f);

		final Burst burst0 = new Burst();
		burst0.setAmount(30);
		burst0.setTimeAtBurst(5f);
		particleSystem.addBurst(burst0);

		particleSystem.play();

		return sobDragon;
	}

	private boolean surfaceDimKnown = false;

	@Override
    public void onDrawFrame(GL10 unused) {
		if (this.surfaceDimKnown) {
	        SceneConductor.getInstance().drawFrame();
		}
    }

	@Override
    public void onSurfaceChanged(GL10 unused, int width, int height) {
    	this.surfaceDimKnown = true;

		final RenderTexture camRenderTex0 = new RenderTexture();
		camRenderTex0.init(width, height);
		camRenderTex0.create();
		this.camSob0.getSharedCamera().setRenderTexture(camRenderTex0);

//		final RenderTexture camRenderTex1 = new RenderTexture();
//		camRenderTex1.init(width, height);
//		camRenderTex1.create();
//		this.camSob1.getSharedCamera().setRenderTexture(camRenderTex1);
//
//		final Material mat = this.cubeCam0.getSharedRenderer().getSharedMaterial(0);
//		mat.setShaderTexture("mainTex", camRenderTex1);

		SurfaceInfo.getInstance().setSurfaceSize(width, height);

		SceneConductor.getInstance().onRenderSurfaceChanged(width, height);
    }
}
